# chdebug start
# Christian Dieterich  Thu Sep 16 06:44:02 CDT 2004
#
# The class Distribution_with_swig implements a Distribution class for
# distutils that runs the build_ext_with_swig class instead of the
# standard build_ext class of distutils.
#
# The class build_ext_with_swig implements a build_ext class for
# distutils that substitutes the functions build_extension and
# swig_sources. The new build_extension creates the build_lib directory
# before running the new swig_sources and that one runs swig with
# include directories and macro definitions. Remember that -U is not
# recognized by swig. swig_sources also copies the Python output of swig
# to the build_lib directory.
#
from distutils.command.build_ext import *
class build_ext_with_swig (build_ext):
# chdebug stop

    def build_extension(self, ext):
        sources = ext.sources
        if sources is None or type(sources) not in (ListType, TupleType):
            raise DistutilsSetupError, \
                  ("in 'ext_modules' option (extension '%s'), " +
                   "'sources' must be present and must be " +
                   "a list of source filenames") % ext.name
        sources = list(sources)

        fullname = self.get_ext_fullname(ext.name)
        if self.inplace:
            # ignore build-lib -- put the compiled extension into
            # the source tree along with pure Python modules

            modpath = string.split(fullname, '.')
            package = string.join(modpath[0:-1], '.')
            base = modpath[-1]

            build_py = self.get_finalized_command('build_py')
            package_dir = build_py.get_package_dir(package)
            ext_filename = os.path.join(package_dir,
                                        self.get_ext_filename(base))
        else:
            ext_filename = os.path.join(self.build_lib,
                                        self.get_ext_filename(fullname))
        depends = sources + ext.depends
        if not (self.force or newer_group(depends, ext_filename, 'newer')):
            log.debug("skipping '%s' extension (up-to-date)", ext.name)
            return
        else:
            log.info("building '%s' extension", ext.name)

        # First, scan the sources for SWIG definition files (.i), run
        # SWIG on 'em to create .c files, and modify the sources list
        # accordingly.
        # chdebug start
        # Christian Dieterich  Thu Sep 16 06:44:02 CDT 2004
        # DIRTY HACK: Create the build_lib directory if it doesn't exist
        self.mkpath(os.path.dirname(ext_filename))
        sources = self.swig_sources(sources, \
                                    os.path.dirname(ext_filename), \
                                    ext.include_dirs, ext.define_macros)
        # chdebug stop

        # Next, compile the source code to object files.

        # XXX not honouring 'define_macros' or 'undef_macros' -- the
        # CCompiler API needs to change to accommodate this, and I
        # want to do one thing at a time!

        # Two possible sources for extra compiler arguments:
        #   - 'extra_compile_args' in Extension object
        #   - CFLAGS environment variable (not particularly
        #     elegant, but people seem to expect it and I
        #     guess it's useful)
        # The environment variable should take precedence, and
        # any sensible compiler will give precedence to later
        # command line args.  Hence we combine them in order:
        extra_args = ext.extra_compile_args or []

        macros = ext.define_macros[:]
        for undef in ext.undef_macros:
            macros.append((undef,))

        objects = self.compiler.compile(sources,
                                        output_dir=self.build_temp,
                                        macros=macros,
                                        include_dirs=ext.include_dirs,
                                        debug=self.debug,
                                        extra_postargs=extra_args,
                                        depends=ext.depends)

        # XXX -- this is a Vile HACK!
        #
        # The setup.py script for Python on Unix needs to be able to
        # get this list so it can perform all the clean up needed to
        # avoid keeping object files around when cleaning out a failed
        # build of an extension module.  Since Distutils does not
        # track dependencies, we have to get rid of intermediates to
        # ensure all the intermediates will be properly re-built.
        #
        self._built_objects = objects[:]

        # Now link the object files together into a "shared object" --
        # of course, first we have to figure out all the other things
        # that go into the mix.
        if ext.extra_objects:
            objects.extend(ext.extra_objects)
        extra_args = ext.extra_link_args or []

        # Detect target language, if not provided
        language = ext.language or self.compiler.detect_language(sources)

        self.compiler.link_shared_object(
            objects, ext_filename,
            libraries=self.get_libraries(ext),
            library_dirs=ext.library_dirs,
            runtime_library_dirs=ext.runtime_library_dirs,
            extra_postargs=extra_args,
            export_symbols=self.get_export_symbols(ext),
            debug=self.debug,
            build_temp=self.build_temp,
            target_lang=language)


    # chdebug start
    def swig_sources (self, sources, output_dir, include_dirs, macros):
    # chdebug stop

        """Walk the list of source files in 'sources', looking for SWIG
        interface (.i) files.  Run SWIG on all that are found, and
        return a modified 'sources' list with SWIG source files replaced
        by the generated C (or C++) files.
        """

        new_sources = []
        swig_sources = []
        swig_targets = {}

        # XXX this drops generated C/C++ files into the source tree, which
        # is fine for developers who want to distribute the generated
        # source -- but there should be an option to put SWIG output in
        # the temp dir.

        if self.swig_cpp:
            target_ext = '.cpp'
        else:
            target_ext = '.c'

        for source in sources:
            (base, ext) = os.path.splitext(source)
            if ext == ".i":             # SWIG interface file
                new_sources.append(base + '_wrap' + target_ext)
                swig_sources.append(source)
                swig_targets[source] = new_sources[-1]
            else:
                new_sources.append(source)

        if not swig_sources:
            return new_sources

        # chdebug start
        # Christian Dieterich  Thu Sep 16 06:44:02 CDT 2004
        # Collect the compiler include directories and the compiler
        # macro definitions into a string
        swig_args = ''
        for macro in self.compiler.macros:
            swig_args += " -D" + macro[0]
            try:
                swig_args += "=" + macro[1]
            except:
                pass
        for macro in macros:
            swig_args += " -D" + macro[0]
            try:
                swig_args += "=" + macro[1]
            except:
                pass
        for include in self.compiler.include_dirs:
            swig_args += " -I" + include
        for include in include_dirs:
            swig_args += " -I" + include
        # chdebug stop
        swig = self.find_swig()
        swig_cmd = [swig, "-python"]
        # chdebug start
        # Christian Dieterich  Thu Sep 16 06:44:02 CDT 2004
        # Append the compiler include directories and the compiler
        # macro definitions to the swig command
        swig_cmd.extend(swig_args.split())
        # chdebug stop
        if self.swig_cpp:
            swig_cmd.append("-c++")

        for source in swig_sources:
            target = swig_targets[source]
            log.info("swigging %s to %s", source, target)
            self.spawn(swig_cmd + ["-o", target, source])
            # chdebug start
            # Christian Dieterich  Thu Sep 16 06:44:02 CDT 2004
            # DIRTY HACK: Copy the Python output to the build_lib directory
            (base, ext) = os.path.splitext(source)
            try:
                self.copy_file(base + ".py", output_dir)
            except:
                pass
            # chdebug stop

        return new_sources

    # swig_sources ()

# class build_ext
